import { Component, OnDestroy } from '@angular/core';
import { Router, NavigationStart, NavigationEnd, NavigationCancel, NavigationError, RoutesRecognized } from '@angular/router';
import { Subscription } from 'rxjs/Subscription';

@Component({
  selector: 'leftmenu',
  templateUrl: './leftmenu.component.html',
  styleUrls: ['./leftmenu.component.css', '../plugin.css']
})
export class LeftMenuComponent implements OnDestroy {
  ngOnDestroy(): void {
    if (this.routerlistener) {
      this.routerlistener.unsubscribe();
    }
  }
  routerlistener: Subscription;
  menus: Array<any> = [
    {
      title: "资源管理1111",
      show: true,
      mainRouter: '/home/resource',
      src: './assets/img/ziyuanguanli.png',
      srcSelect: '/assets/img/ziyuanguanli-select.jpg',
      submenus: [
        {
          title: "物理机管理",
          router: 'psMachine',
        },
        {
          title: "云主机管理",
          router: 'cloudMachine',
        },
        {
          title: "租户管理",
          router: 'tenants',
        }
      ]
    },
    {
      title: "告警与监控",
      show: false,
      mainRouter: '/home/warn',
      src: '/assets/img/jiankong.jpg',
      srcSelect: '/assets/img/jiankong-select.png',
      submenus: [
        {
          title: "监控管理",
          router: 'watching/psmachine',
          matchRouter: ['watching/cloudmachine']
        },
        {
          title: "告警管理",
          router: 'warnning/notprocess',
          matchRouter: ['warnning/processed']
        }
      ]
    },
    {
      title: "工作流",
      show: false,
      src: '/assets/img/gongzuliu.jpg',
      srcSelect: '/assets/img/gongzuliu-select.png',
      mainRouter: '/home/workflow',
      submenus: [
        {
          title: "订单管理",
          router: 'order',
        },
        {
          title: "审批管理",
          router: 'approval',
        },
        {
          title: "工作流管理",
          router: 'procedure',
        }
      ]
    }
  ]
  constructor(private router: Router) {
    this.routerlistener = router.events.subscribe(event => {
      if (event instanceof NavigationStart) {
        //不处理
      } else if (event instanceof NavigationEnd) {
        let url = event.url;
        let foundMenu, foundSubMenu;
        for (let menu of this.menus) {
          for (let submenu of menu.submenus) {
            if (menu.mainRouter + '/' + submenu.router == url) {
              foundMenu = menu;
              foundSubMenu = submenu;
            }
            if (foundMenu == null && foundSubMenu == null && submenu.matchRouter) {
              for (let r of submenu.matchRouter) {
                if (menu.mainRouter + '/' + r == url) {
                  foundMenu = menu;
                  foundSubMenu = submenu;
                }
              }
            }

            submenu.active = false;
          }
        }

        if (foundMenu && foundSubMenu) {
          foundMenu.show = true;
          foundSubMenu.active = true;
        }

      } else if (event instanceof NavigationCancel) {
        //不处理
      } else if (event instanceof NavigationError) {
        //不处理
      } else if (event instanceof RoutesRecognized) {
        //不处理
      }
    });
  }

  menuClick(menu) {
    menu.show = !menu.show;
  }

  subMenuClick(menu, submenu) {
    this.router.navigate([menu.mainRouter + '/' + submenu.router]);
  }
}
