import {
  Component,
  Input,
  OnInit,
  Output,
  EventEmitter,
  OnChanges,
  SimpleChanges,
  ViewChild,
  ViewContainerRef,
  ComponentFactoryResolver
} from '@angular/core';
import {Router, NavigationStart, NavigationEnd, NavigationCancel, NavigationError, RoutesRecognized} from '@angular/router';

@Component({
  selector: 'dc-node',
  templateUrl: './node.component.html',
  styleUrls: ['./tree.component.css', '../plugin.css']
})
export class NodeComponent implements OnInit, OnChanges {
  showSub: any;
  _nodeData;
  @Input() root: boolean;
  @Input() keyId: string;
  @Input() keyName: string;
  @Input() keyChild: string;
  @Input() options: any;
  @Output() clickEvent = new EventEmitter();
  @Output() extendEvent = new EventEmitter();
  iconClsFolder: string;
  iconClsExpand: string;
  iconClsFile: string;


  @Input() set nodeData(v) {
    this._nodeData = v;
    this.init();
  }

  get nodeData() {
    return this._nodeData;
  }

  @Output() nodeChangeEvent = new EventEmitter();


  constructor(private resolver: ComponentFactoryResolver, private router: Router) {

  }

  init(): any {
    if (this.needShow(this.nodeData)) {
      this.nodeData.show = true;
    }
  }

  private needShow(node: any): any {
    if (node.show) {
      return true;
    }
    if (node.subNode == null || node.subNode.length == 0) {
      return false;
    }
    let result = false;
    for (let n of node.subNode) {
      if (this.needShow(n)) {
        result = true;
      }
    }
    return result;
  }

  ngOnInit(): void {
    this.initClsObj();
  }

  initClsObj() {
    if (this.options && this.options.iconCls) {
      this.iconClsFolder = this.options.iconCls.folder;
      this.iconClsFile = this.options.iconCls.file;
      this.iconClsExpand = this.options.iconCls.expand;
    }
  }

  getFileIcon() {
    if (this.iconClsFile && (!this.nodeData[this.keyChild] || this.nodeData[this.keyChild].length == 0)) {
      return this.iconClsFile;
    }
    return '';
  }

  getFolderIcon() {
    if (this.iconClsFolder && this.nodeData[this.keyChild] && this.nodeData[this.keyChild].length > 0) {
      return this.iconClsFolder;
    }
    return '';
  }

  getExpandIcon() {
    if (this.iconClsExpand && this.nodeData.show) {
      return this.iconClsExpand;
    }
    return '';
  }

  clickNode(nodeData) {
    this.clickEvent.emit(this.nodeData);
  }

  changeShow() {
    this.nodeData.show = !this.nodeData.show;
    this.extendEvent.emit(this.nodeData);
  }

  doNodeFn(option) {
    if (option.fn) {
      option.fn(this.nodeData);
    }
  }

  _clickNode(event) {
    this.clickEvent.emit(event);
  }

  _extendNode(event) {
    this.extendEvent.emit(event);
  }

  ngOnChanges(changes: SimpleChanges) {
  }


  getText() {
    if (this._nodeData.show) {
      return '-';
    } else {
      return '+';
    }
  }
}
