import {
  Component,
  Input,
  OnInit,
  Output,
  EventEmitter,
  OnChanges,
  SimpleChanges,
  ViewChild,
  ViewContainerRef,
  ComponentFactoryResolver
} from '@angular/core';
import {Router, NavigationStart, NavigationEnd, NavigationCancel, NavigationError, RoutesRecognized} from '@angular/router';

@Component({
  selector: 'dc-tree',
  templateUrl: './tree.component.html',
  styleUrls: ['./tree.component.css', '../plugin.css']
})
export class TreeComponent implements OnInit {
  @Output() clickEvent = new EventEmitter();
  @Output() extendEvent = new EventEmitter();
  @Input() options: any;
  _treeDatas;
  @Input() set treeDatas(v) {
    this._treeDatas = v;
  }

  get treeDatas() {
    return this._treeDatas;
  }

  tempSearch = [];
  selectedNode;

  @Input() keyId;
  @Input() keyName;
  @Input() keyChild;

  constructor(private resolver: ComponentFactoryResolver, private router: Router) {

  }

  ngOnInit(): void {
    if (!this.keyId) {
      this.keyId = 'id';
    }
    if (!this.keyName) {
      this.keyName = 'title';
    }
    if (!this.keyChild) {
      this.keyChild = 'subNode';
    }
  }


  clickNode(event) {
    this.clickEvent.emit(event);
  }

  extendNode(event) {
    this.extendEvent.emit(event);
  }

  private _findNodeById(id, source) {
    for (let i = 0; i < source.length; i++) {
      const theI = source[i];
      if (theI[this.keyId] == id) {
        return theI;
      }
      if (theI[this.keyChild]) {
        let res = this._findNodeById(id, theI[this.keyChild]);
        if (res) {
          return res;
        }
      }
    }
  }

  private _findNodeByAllName(name, source) {
    for (let i = 0; i < source.length; i++) {
      const theI = source[i];
      if (theI[this.keyName] == name) {
        this.tempSearch.push(theI);
      }
      if (theI[this.keyChild]) {
        this._findNodeByAllName(name, theI[this.keyChild]);
      }
    }
  }

  private _findNodeByName(name, source) {
    for (let i = 0; i < source.length; i++) {
      let theI = source[i];
      if (theI[this.keyName].toLowerCase().indexOf(name.toLowerCase()) != -1) {
        this.tempSearch.push(theI);
      }
      if (theI[this.keyChild]) {
        this._findNodeByName(name, theI[this.keyChild]);
      }
    }
  }

  private _findParentNodeById(id, source, parent?) {
    for (let i = 0; i < source.length; i++) {
      let theI = source[i];
      if (theI[this.keyId] == id) {
        return parent;
      }
      if (theI[this.keyChild]) {
        let res = this._findParentNodeById(id, theI[this.keyChild], theI);
        if (res) {
          return res;
        }
      }
    }
  }

  private _findParentsNodeById(result, id, source) {
    for (let i = 0; i < source.length; i++) {
      let theI = source[i];
      if (theI[this.keyId] == id) {
        result.push(theI);
        return theI;
      }
      if (theI[this.keyChild]) {
        let res = this._findParentsNodeById(result, id, theI[this.keyChild]);
        if (res) {
          result.push(theI);
          return res;
        }
      }
    }
  }

  private _findChildrensNodeByNode(result, source) {
    result.push(source);
    if (source[this.keyChild]) {
      for (let i = 0; i < source[this.keyChild].length; i++) {
        this._findChildrensNodeByNode(result, source[this.keyChild][i]);
      }
    }
  }

  // 匹配搜索
  private _matchSearchNodesByKey(result, val, key, source) {
    for (let i = 0; i < source.length; i++) {
      let theI = source[i];
      if (theI[key] == val) {
        result.push(source[i]);
      }
      if (theI[this.keyChild]) {
        this._matchSearchNodesByKey(result, val, key, theI[this.keyChild]);
      }
    }
  }

  // 模糊搜索
  private _fuzzySearchNodesByKey(result, val, key, source) {
    for (let i = 0; i < source.length; i++) {
      let theI = source[i];
      if (theI[key].toLowerCase().indexOf(val.toLowerCase()) != -1) {
        result.push(source[i]);
      }
      if (theI[this.keyChild]) {
        this._fuzzySearchNodesByKey(result, val, key, theI[this.keyChild]);
      }
    }
  }

  // 根据ID查询节点
  findNodeById(id) {
    this.tempSearch = [];
    return this._findNodeById(id, this.treeDatas);
  }

  // 根据名称查询全匹配节点
  findNodeByName(name) {
    this.tempSearch = [];
    this._findNodeByName(name, this.treeDatas);
    return this.tempSearch;
  }

  // 根据名称模糊查询节点
  findNodeByAllName(name) {
    this.tempSearch = [];
    this._findNodeByAllName(name, this.treeDatas);
    return this.tempSearch;
  }

  // 获取父节点
  findParentNodeById(id) {
    return this._findParentNodeById(id, this.treeDatas);
  }

  //获取父节点链
  findParentsNodeById(id) {
    let result = [];
    this._findParentsNodeById(result, id, this.treeDatas);
    return result;
  }

  findChildrensNodeByNode(node) {
    let result = [];
    this._findChildrensNodeByNode(result, node);
    return result;
  }

  // 插入节点
  insertNode(child, parent?) {
    let targetData;
    let parents;
    if (parent) {
      parents = this.findParentsNodeById(parent[this.keyId]);
      if (parents && parents.length > 0) {
        if (!parents[0][this.keyChild]) {
          parents[0][this.keyChild] = [];
        }
        targetData = parents[0][this.keyChild];
        for (let i = 0; i < parents.length; i++) {
          parents[i].show = true;
        }
      }
    } else { // 在根节点下插入
      targetData = this.treeDatas;
    }
    targetData.push(child);
    return child;
  }

  //更新节点数据
  updateNodeData(data, keys) {
    if (!keys) {
      return;
    }
    let targetNode = this.findNodeById(data[this.keyId]);
    for (let i = 0; i < keys.length; i++) {
      targetNode[keys] = data[keys];
    }
  }

  //删除节点
  deleteNodeById(id) {
    let parentNode = this.findParentNodeById(id);
    if (!parentNode) {
      parentNode = this.treeDatas;
    } else {
      parentNode = parentNode[this.keyChild];
    }
    for (let i = 0; i < parentNode.length; i++) {
      if (parentNode[i][this.keyId] == id) {
        parentNode.splice(i, 1);
        break;
      }
    }
  }

  // 匹配搜索
  matchSearchNodesByKey(val, key) {
    let result = [];
    this._matchSearchNodesByKey(result, val, key, this.treeDatas);
    return result;
  }

  // 模糊搜索
  fuzzySearchNodesByKey(val, key) {
    let result = [];
    this._fuzzySearchNodesByKey(result, val, key, this.treeDatas);
    return result;
  }

  //展开节点
  expandNode(node) {
    let parents = this.findParentsNodeById(node[this.keyId]);
    if (parents && parents.length > 0) {
      if (!parents[0][this.keyChild]) {
        parents[0][this.keyChild] = [];
      }
      for (let i = 0; i < parents.length; i++) {
        parents[i].show = true;
      }
    }
  }

  // 展开节点s
  expandNodes(nodes) {
    for (let i = 0; i < nodes.length; i++) {
      this.expandNode(nodes[i]);
    }
  }

  // 高亮节点s
  lighthightNodes(nodes) {
    for (let i = 0; i < nodes.length; i++) {
      nodes[i].lighthight = true;
    }
  }

  // 移除高亮s
  removeLighthightNodes(nodes) {
    for (let i = 0; i < nodes.length; i++) {
      delete nodes[i].lighthight;
    }
  }

  // 选中节点
  selectTreeNode(node) {
    this.selectedNode && this.deSelectTreeNode(this.selectedNode);
    let item = this.findNodeById(node[this.keyId]);
    if (item) {
      item.selected = true;
      this.selectedNode = item;
    }
  }

  // 移除选中
  deSelectTreeNode(node) {
    let item = this.findNodeById(node[this.keyId]);
    if (item) {
      delete item.selected;
    }
  }
}
